--- clamav-0.70-rc/clamav-milter/clamav-milter.c	Mon Mar 15 15:03:13 2004
+++ clamav-0.70-rc-jm/clamav-milter/clamav-milter.c	Tue Mar 30 18:29:39 2004
@@ -566,6 +566,7 @@
 	char	*filename;	/* Where to store the message in quarantine */
 	u_char	*body;		/* body of the message if Sflag is set */
 	size_t	bodyLen;	/* number of bytes in body */
+	size_t	nWritten;	/* number of bytes we have written */
 	header_list_t headers;	/* Message headers */
 };
 
@@ -1914,17 +1915,14 @@
 clamfi_body(SMFICTX *ctx, u_char *bodyp, size_t len)
 {
 	struct privdata *privdata = (struct privdata *)smfi_getpriv(ctx);
+	struct cfgstruct *cpt = NULL;	
+	size_t sendlen = 0;
 
 	if(logVerbose)
 		syslog(LOG_DEBUG, "clamfi_envbody: %u bytes", len);
 #ifdef	CL_DEBUG
 	printf("clamfi_envbody: %u bytes\n", len);
 #endif
-
-	if(clamfi_send(privdata, len, (char *)bodyp) < 0) {
-		clamfi_cleanup(ctx);
-		return cl_error;
-	}
 	if(Sflag) {
 		if(privdata->body) {
 			assert(privdata->bodyLen > 0);
@@ -1938,6 +1936,41 @@
 			privdata->bodyLen = len;
 		}
 	}
+
+	if((!quarantine_dir) 
+		&& (cpt = cfgopt(copt, "StreamMaxLength")) 
+		&& cpt->numarg < (len + privdata->nWritten)
+	   ){
+		sendlen = (cpt->numarg - privdata->nWritten);
+		if(use_syslog && privdata->nWritten != cpt->numarg){
+			char buf[1024];	
+			strncpy(buf,privdata->from,sizeof(buf));
+			syslog(LOG_INFO,"Stream from %s size exceeded max of %u , already wrote %u, will write %u more instead of len %u",
+					buf,
+					cpt->numarg,
+					privdata->nWritten,
+					sendlen,
+					len); 
+		}
+	}
+	else {
+		sendlen = len;
+	}
+
+	if(!sendlen)
+		return SMFIS_CONTINUE;
+
+
+	if(sendlen && (clamfi_send(privdata, sendlen, (char *)bodyp) < 0)) {
+		clamfi_cleanup(ctx);
+		return cl_error;
+	}else
+	{
+		if(sendlen)
+			privdata->nWritten += sendlen;
+	}		
+
+
 	return SMFIS_CONTINUE;
 }
 
@@ -2389,6 +2422,7 @@
 		printf("clamfi_send: len=%u bufsiz=%u\n", len, sizeof(output));
 #endif
 
+	errno = 0;
 	while(len > 0) {
 		const int nbytes = (quarantine_dir) ?
 			write(privdata->dataSocket, ptr, len) :
@@ -2400,7 +2434,8 @@
 			perror("send");
 			checkClamd();
 			if(use_syslog)
-				syslog(LOG_ERR, "write failure to clamd");
+				syslog(LOG_ERR, "write failure to clamd, nbytes: %d, quarantine_dir: %s, error: %s", 
+						nbytes, quarantine_dir, strerror(errno) );
 
 			return -1;
 		}
